
import io
import uuid
from unittest.mock import patch

from odoo.tests import tagged, new_test_user
from odoo.tools import DotDict
from odoo.tools.pdf import OdooPdfFileReader

from odoo.addons.base.tests.common import HttpCaseWithUserPortal


@tagged("post_install", "-at_install")
class TestPages(HttpCaseWithUserPortal):
    quotation_template = "openerp_enterprise.quotation_template_odoo_yearly"

    def _mock_get_most_valid_subscription(self, valid_subscriptions_code):
        def _get_subscription(code):
            if code in valid_subscriptions_code:
                # because `has_enterprise_download` is defined in another module
                # use a DotDict to simlate a record, which is sufficient for our
                # test case.
                fake_subscription_vals = {
                    "id": 42, "client_order_ref": code,
                    "state": "sale", "subscription_state": "3_progress",
                    "enterprise_has_download": True,
                }
                if isinstance(valid_subscriptions_code, dict):
                    fake_subscription_vals.update(valid_subscriptions_code[code])
                return DotDict(fake_subscription_vals)
            return DotDict()
        return patch.object(
            type(self.env["sale.order"]), "_get_most_valid_subscription",
            wraps=_get_subscription, create=True,
        )

    def test_page_security_advisories(self):
        with self._mock_get_most_valid_subscription(["M2022SECADV1"]):
            self.start_tour("/", "security_advisories_list")

    def test_page_security_compliance_reports(self):
        self.start_tour("/", "security_compliance_reports")

    def test_security_compliance_watermark(self):
        watermark_login = "portal-watermark-%s" % (uuid.uuid4())
        new_test_user(self.env, login=watermark_login, groups="base.group_portal")
        self.authenticate(watermark_login, watermark_login)
        response = self.url_open(
            "/security/compliance-reports/2021-12-01-ISAE3402 SOC 1 Type I.pdf",
            timeout=30,  # watermarking could take some times
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers["Content-Type"], "application/pdf")

        # Ensure the watermark is present on each pages
        pdf_data = OdooPdfFileReader(io.BytesIO(response.content))
        for page_num in range(pdf_data.getNumPages()):
            page_text = pdf_data.getPage(page_num).extractText()
            self.assertIn(watermark_login, page_text, "Watermark is not present on page %d" % (page_num + 1,))

    def test_page_download_community(self):
        self.start_tour("/", "page_download_community")

        last_lead = self.env["crm.lead"].search([], order="id DESC", limit=1)
        self.assertEqual(last_lead.partner_name, "My Business Company")
        self.assertEqual(last_lead.contact_name, "John Download")
        self.assertEqual(last_lead.email_from, "john@mybusinesscompany.example.org")

    def test_page_download_enterprise_as_anonymous(self):
        test_subscriptions = {
            # valid subscription
            "M2022DLCODETEST": {},
            # expired subscription
            "M2022DLCODEEXPIRED": {
                "state": "cancel",
                "subscription_state": "6_churn",
                "enterprise_has_download": False,
            }
        }
        with self._mock_get_most_valid_subscription(test_subscriptions):
            self.start_tour("/", "page_download_enterprise_anonymous")

    def test_page_download_enterprise_as_portal(self):
        with self._mock_get_most_valid_subscription(["M2022DLCODETEST"]):
            self.start_tour("/", "page_download_enterprise_code", login="portal")
